Global $iEscaliersTmp

; #FUNCTION# ====================================================================================================================
; Name...........: _CreeLab3D
; Description ...: Cree un labyrinthe
; Syntax.........: _CreeLab3D($iLargeur, $iHauteur, $iEtages, $iEscaliers, ByRef $aMursHorizontaux, ByRef $aMursVerticaux, ByRef $aObjets)
; Parameters ....: $iLargeur         - Largeur du labyrinthe
;                  $iHauteur         - Hauteur du labyrinthe
;                  $iEtages          - Nombre d'tages du labyrinthe
;                  $iEscaliers       - Nombre d'escaliers par tage
;                  $aMursHorizontaux - Variable qui contiendra les murs horizontaux
;                  $aMursVerticaux   - Variable qui contiendra les murs verticaux
;                  $aObjets          - Variable qui contiendra les objets (escaliers)
; Return values .:
; Author ........: Tolf
; Modified.......:
; Remarks .......: Le labyrinthe est gnr a partir de l'algorithme dcrit sur la page http://ilay.org/yann/articles/maze/
;				 |dans le paragraphe "Unicit de chemin et ala"
; Related .......:
; Link ..........;
; Example .......;
; ===============================================================================================================================
Func _CreeLab3D()
	Local $aCellules[$iLargeur][$iHauteur], $i, $j, $k
	$iEscaliersTmp = $iEscaliers
	
	; On initialise les tables
	Dim $aMursHorizontaux[$iLargeur][$iHauteur - 1][$iEtages]
	For $i = 0 To $iLargeur - 1
		For $j = 0 To $iHauteur - 2
			For $k = 0 To $iEtages - 1
				$aMursHorizontaux[$i][$j][$k] = 1
			Next
		Next
	Next
	Dim $aMursVerticaux[$iLargeur - 1][$iHauteur][$iEtages]
	For $i = 0 To $iLargeur - 2
		For $j = 0 To $iHauteur - 1
			For $k = 0 To $iEtages - 1
				$aMursVerticaux[$i][$j][$k] = 1
			Next
		Next
	Next
	
	Local $Z
	For $Z = 0 To $iEtages - 1
		For $i = 0 To $iLargeur - 1
			For $j = 0 To $iHauteur - 1
				$aCellules[$i][$j] = $i * $iHauteur + $j
			Next
		Next
		If $Z < $iEtages - 1 Then CreeEscaliers($Z, $aCellules)
		
		If $Z = $iEtages - 1 Then $iEscaliersTmp = 1
		CreeEtage($Z, $aCellules)
	Next
	
	Return SetError(0, 0, 1)
EndFunc   ;==>_CreeLab3D


Func CreeEtage($Z, $aCellules)
	; On cre le labyrinthe en enlevant un  un des murs
	Local $X1, $Y1, $X2, $Y2, $randDir, $valCell1, $valCell2, $nbreMurs = 0
	While 1
		$randDir = Random(1, 2, 1) ; On choisit une des 2 directions possibles (horizontal / vertical)
		
		Switch $randDir
			Case 1 ; mur horizontal
				$X1 = Random(0, $iLargeur - 1, 1) ; Coordonnes alatoires
				$Y1 = Random(0, $iHauteur - 2, 1)
				If $aMursHorizontaux[$X1][$Y1][$Z] = 0 Then ContinueLoop
				$X2 = $X1
				$Y2 = $Y1 + 1
			Case 2 ; mur vertical
				$X1 = Random(0, $iLargeur - 2, 1) ; Coordonnes alatoires
				$Y1 = Random(0, $iHauteur - 1, 1)
				If $aMursVerticaux[$X1][$Y1][$Z] = 0 Then ContinueLoop
				$X2 = $X1 + 1
				$Y2 = $Y1
		EndSwitch
		
		; On obtient les valeurs des deux cases
		$valCell1 = $aCellules[$X1][$Y1]
		$valCell2 = $aCellules[$X2][$Y2]
		
		If $valCell1 = $valCell2 Then ContinueLoop ; Si les valeurs sont identiques, les 2 cases sont dj relies
		
		; On enlve le mur
		Switch $randDir
			Case 1
				$aMursHorizontaux[$X1][$Y1][$Z] = 0
			Case 2
				$aMursVerticaux[$X1][$Y1][$Z] = 0
		EndSwitch
		
		; On met la mme valeur dans les cases de la chane
		Switch $valCell2
			Case - 1 ; La valeur -1 est prioritaire car les escaliers montants ne doivent pas tre relis
				NettoieCellules($aCellules, $X1, $Y1, $valCell2, $valCell1)
			Case Else
				NettoieCellules($aCellules, $X2, $Y2, $valCell1, $valCell2)
		EndSwitch
		
		; On a enlev 1 mur de plus
		$nbreMurs += 1
		If $nbreMurs = ($iLargeur * $iHauteur) - $iEscaliersTmp Then ExitLoop
	WEnd
EndFunc   ;==>CreeEtage


Func CreeEscaliers($Z, ByRef $aCellules)
	Local $X, $Y, $i, $j, $iContinuer, $ubObjets = UBound($aObjets)
	
	For $i = 1 To $iEscaliersTmp
		While 1
			$iContinuer = 1
			$X = Random(0, $iLargeur - 1, 1)
			$Y = Random(0, $iHauteur - 1, 1)
			; On vrifie que la case n'est pas dj occupe par un objet
			For $j = 0 To $ubObjets - 1
				If $aObjets[$j][1] = $X And $aObjets[$j][2] = $Y And $aObjets[$j][3] = $Z Then $iContinuer = 0
			Next
			If $iContinuer = 1 Then ExitLoop
		WEnd
		
		ReDim $aObjets[$ubObjets + 2][4]
		; Escalier montant
		$aObjets[$ubObjets][0] = "M"
		$aObjets[$ubObjets][1] = $X
		$aObjets[$ubObjets][2] = $Y
		$aObjets[$ubObjets][3] = $Z
		
		; Escalier descandant
		$aObjets[$ubObjets + 1][0] = "D"
		$aObjets[$ubObjets + 1][1] = $X
		$aObjets[$ubObjets + 1][2] = $Y
		$aObjets[$ubObjets + 1][3] = $Z + 1
		
		$ubObjets += 2
		
		$aCellules[$X][$Y] = -1 ; Tous les escaliers montants ont la mme valeur : ils ne sont donc pas relis
	Next
EndFunc   ;==>CreeEscaliers


Func NettoieCellules(ByRef $aCellules, $X2, $Y2, $valCell1, $valCell2)
	$aCellules[$X2][$Y2] = $valCell1
	
	If $X2 > 0 And $aCellules[$X2 - 1][$Y2] = $valCell2 Then NettoieCellules($aCellules, $X2 - 1, $Y2, $valCell1, $valCell2)
	If $X2 < $iLargeur - 1 And $aCellules[$X2 + 1][$Y2] = $valCell2 Then NettoieCellules($aCellules, $X2 + 1, $Y2, $valCell1, $valCell2)
	If $Y2 > 0 And $aCellules[$X2][$Y2 - 1] = $valCell2 Then NettoieCellules($aCellules, $X2, $Y2 - 1, $valCell1, $valCell2)
	If $Y2 < $iHauteur - 1 And $aCellules[$X2][$Y2 + 1] = $valCell2 Then NettoieCellules($aCellules, $X2, $Y2 + 1, $valCell1, $valCell2)
EndFunc   ;==>NettoieCellules